/*
 * (c) Copyright 2000-2012 spring saarlouis
 * All Rights Reserved.
 *
 * contact info@spring.de for more information
 */
package de.spring.mobile;

import android.content.Context;
import android.graphics.Point;
import android.os.Build;
import android.util.Log;
import android.view.WindowManager;
import android.widget.VideoView;

import com.google.android.gms.cast.framework.CastContext;
import com.google.android.gms.cast.framework.SessionManagerListener;
import com.google.android.gms.cast.framework.CastSession;
import com.google.android.gms.common.api.Status;
import com.google.android.gms.common.api.ResultCallback;

import org.json.JSONObject;

import java.io.IOException;
import java.lang.ref.WeakReference;

/**
 * StreamAdapter implementation with Chromecast support.
 * Implementation of an adapter for the {@link VideoView} with standard
 * android api and support for Chromecast. The video can be played locally or casted to a Chromecast device.
 * Its also possible, to switch between local and remote playback while playing the video.
 *
 * @author <a href="mailto:support@spring.de">spring GmbH &amp; Co.KG</a>
 */
public class ChromeCastVideoViewAdapter implements StreamAdapter, SessionManagerListener<CastSession> {

	private CastSession castSession;
	private WeakReference<VideoView> videoViewWeakReference;
	private final KMAChromecastChannel kmaChromecastChannel = new KMAChromecastChannel();

	/**
	 * Create the adapter with the given {@link VideoView} and the given {@link CastContext}.
	 *
	 * @param videoView The video view
	 * @param castContext The chromecast context
	 *
	 * @throws NullPointerException If the parameter <code>videoView</code> or the parameter <code>castContext</code> is <code>null</code>.
	 */
	public ChromeCastVideoViewAdapter(VideoView videoView, CastContext castContext) {
		if (videoView == null) {
			throw new NullPointerException("videoView may not be null");
		} else if (castContext == null) {
			throw new NullPointerException("castContext may not be null");
		} else {
			this.videoViewWeakReference = new WeakReference(videoView);
			this.castSession = castContext.getSessionManager().getCurrentCastSession();
			castContext.getSessionManager().addSessionManagerListener(this, CastSession.class);
			if (this.castSession != null && this.castSession.isConnected()) {
				this.onApplicationConnected(this.castSession);
			}
		}
	}

	/**
	 * Returns the meta object.
	 *
	 * @see StreamAdapter#getMeta()
	 */
	@Override
	public Meta getMeta() {
		return new Meta() {

			/**
			 * Returns the string <code>android.widget.VideoView</code>.
			 *
			 * @see Meta#getPlayerName()
			 *
			 * @return The player name <code>android.widget.VideoView</code>.
			 */
			public String getPlayerName() {
				return "android.widget.VideoView";
			}

			/**
			 * Returns the version of the operating system.
			 *
			 * @see Meta#getPlayerVersion()
			 *
			 * @return The system version.
			 */
			public String getPlayerVersion() {
				return Build.VERSION.RELEASE;
			}

			/**
			 * Returns the screen height.
			 *
			 * @see WindowManager#getDefaultDisplay(){@link #getWidth()}
			 * @see Meta#getScreenHeight()
			 */
			public int getScreenHeight() {
				VideoView videoView = videoViewWeakReference.get();
				if (videoView != null) {
					Point point = new Point();
					(((WindowManager)videoViewWeakReference.get().getContext().getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay()).getSize(point);
					return point.y;
				}
				else return 0;
			}

			/**
			 * Returns the screen width.
			 *
			 * @see WindowManager#getDefaultDisplay(){@link #getHeight()}
			 * @see Meta#getScreenWidth()
			 */
			public int getScreenWidth() {
				VideoView videoView = videoViewWeakReference.get();
				if (videoView != null) {
					Point point = new Point();
					(((WindowManager)videoViewWeakReference.get().getContext().getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay()).getSize(point);
					return point.x;
				}
				else return 0;
			}

		};
	}

	/**
	 * Returns the position by calling the method {@link VideoView#getCurrentPosition()}.
	 *
	 * @see StreamAdapter#getPosition()
	 * @see VideoView#getCurrentPosition()
	 *
	 * @return The position in seconds.
	 */
	@Override
	public int getPosition() {
		VideoView videoView = videoViewWeakReference.get();
		if (videoView != null) {
			return (int)Math.round(videoView.getCurrentPosition()/1000.0);
		}
		return 0;
	}

	/**
	 * Returns the duration by calling the method {@link VideoView#getDuration()}.
	 *
	 * @see StreamAdapter#getDuration()
	 * @see VideoView#getCurrentPosition()
	 *
	 * @return The duration in seconds.
	 */
	@Override
	public int getDuration() {
		VideoView videoView = videoViewWeakReference.get();
		if (videoView != null) return (int)Math.round(videoView.getDuration()/1000.0);
		else return 0;
	}

	/**
	 * Returns the width of the {@link VideoView}
	 *
	 * @see VideoView#getWidth()
	 * @see StreamAdapter#getWidth()
	 *
	 * @return The width
	 */
	@Override
	public int getWidth() {
		VideoView videoView = videoViewWeakReference.get();
		if (videoView != null) return videoView.getWidth();
		else return 0;
	}

	/**
	 * Returns the height of the {@link VideoView}
	 *
	 * @see VideoView#getHeight()
	 * @see StreamAdapter#getHeight()
	 *
	 * @return The height
	 */
	@Override
	public int getHeight() {
		VideoView videoView = videoViewWeakReference.get();
		if (videoView != null) return videoView.getHeight();
		else return 0;
	}

	/**
	 * Returns if video is streaming.
	 */
	@Override
	public  boolean isCasting(){
		return  false;
	}

	/**
	 *
	 * @param session Session for managing connections to a Cast receiver device
	 * @param error Error Code
	 */
	@Override
	public void onSessionEnded(CastSession session, int error) {
		this.castSession = session;
	}

	/**
	 *
	 * @param session Session for managing connections to a Cast receiver device
	 * @param wasSuspended Suspend flag
	 */
	@Override
	public void onSessionResumed(CastSession session, boolean wasSuspended) {
		onApplicationConnected(session);
	}

	/**
	 *
	 * @param session Session for managing connections to a Cast receiver device
	 * @param error Error Code
	 */
	@Override
	public void onSessionResumeFailed(CastSession session, int error) {
		this.castSession = session;
	}

	/**
	 *
	 * @param session Session for managing connections to a Cast receiver device
	 * @param sessionId Session's Identifier
	 */
	@Override
	public void onSessionStarted(CastSession session, String sessionId) {
		try {
			session.setMessageReceivedCallbacks(
					kmaChromecastChannel.getNamespace(),
					kmaChromecastChannel);
		} catch (IOException e) {
			Log.e("Error", "Exception while creating channel", e);
		}
		onApplicationConnected(session);
	}

	/**
	 *
	 * @param session Session for managing connections to a Cast receiver device
	 * @param error Error Code
	 */
	@Override
	public void onSessionStartFailed(CastSession session, int error) {
		this.castSession = session;
	}

	/**
	 *
	 * @param session Session for managing connections to a Cast receiver device
	 */
	@Override
	public void onSessionStarting(CastSession session) {
		this.castSession = session;
	}

	/**
	 *
	 * @param session Session for managing connections to a Cast receiver device
	 */
	@Override
	public void onSessionEnding(CastSession session) {
		this.castSession = session;
	}

	/**
	 *
	 * @param session Session for managing connections to a Cast receiver device
	 * @param sessionId Session's Identifier
	 */
	@Override
	public void onSessionResuming(CastSession session, String sessionId) {
		this.castSession = session;
	}

	/**
	 *
	 * @param session Session for managing connections to a Cast receiver device
	 * @param reason Suspend reason's code
	 */
	@Override
	public void onSessionSuspended(CastSession session, int reason) {
		Log.d("VideoViewAdapter", "Chromecast device disconnected");
		this.castSession = session;
	}

	/**
	 *
	 * Called when a connection was setup or resumed
	 *
	 * Gets the Chromecast Id and then sends it to receiver using a channel
	 */
	private void onApplicationConnected(CastSession aCastSession) {
		Log.i("onApplicationConnected", "onApplicationConnected");

		if (kmaChromecastChannel != null) {
			try {
				JSONObject jsonObject = new JSONObject();
				jsonObject.put("id", aCastSession.getCastDevice().getDeviceId());
				aCastSession.sendMessage(kmaChromecastChannel.getNamespace(), jsonObject.toString())
						.setResultCallback(
								new ResultCallback<Status>() {
									@Override
									public void onResult(Status result) {
										if (!result.isSuccess()) {
											Log.e("KMAChromecastChannel", "Sending message failed");
										}
									}
								});
			} catch (Exception e) {
				Log.e(" KMAChromecastChannel", "Exception while sending message", e);
			}
		}
	}

}
